#include "DHT22.h"

DHT22::DHT22(uint8_t pin) : pin_(pin) {
  pinMode(pin_, INPUT_PULLUP);
}

uint32_t pulseIn2(uint8_t pin, uint8_t level, uint32_t timeout) {
  const uint32_t start = micros();
  while (digitalRead(pin) != level) if (micros() - start > timeout) return 0;
  const uint32_t edge = micros();
  while (digitalRead(pin) == level) if (micros() - start > timeout) return 0;
  return (micros() - edge);
}

bool DHT22::read(uint16_t *temperature, uint16_t *humidity) {
  uint8_t chksum = 0;
  uint8_t data[5];
  uint32_t width;

  pinMode(pin_, OUTPUT);
  digitalWrite(pin_, LOW);
  delayMicroseconds(1000);

  pinMode(pin_, INPUT_PULLUP);

  width = pulseIn2(pin_, LOW, 285);
  if (width < 60 || width > 100) goto error;
  width = pulseIn2(pin_, HIGH, 85);
  if (width == 0) goto error;

  for (uint8_t i = 0; i < 5; i++) {
    uint8_t buf = 0;
    for (uint8_t j = 0; j < 8; j++) {
      width = pulseIn2(pin_, HIGH, 140);  // 0: 22-30us, 1: 68-75us
      if (width == 0) goto error;
      buf <<= 1;
      if (width >= 50) buf |= 0x01;
    }
    data[i] = buf;
    chksum += buf;
  }

  if (chksum != (uint8_t)(data[4] + data[4])) goto error;
  *humidity = (((uint16_t)data[0] << 8) | data[1]);
  *temperature = (((uint16_t)data[2] << 8) | data[3]);
  return true;
error:
  return false;
}

